/*
 * Licensed Materials - Property of IBM
 *
 * com.ibm.etools.c
 *
 *  Copyright IBM Corporation 2004. All Rights Reserved. 
 *
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.etools.c.importer;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;

/**
 * The CImporter class extends the MOF ResourceFactory interface and implements
 * its load(ResourceSet, String, InputStream, Object) method. It is registered
 * to be an extension point of com.ibm.etools.emf.extension_parser to handle .c
 * and .h files via plugin.xml.
 * 
 * The single instance of CImporter is created by the plugin mechanism.This
 * class is not intended to be called directly by the user. The importer
 * framework can find out the corresponding importer registered for the file
 * extension and call it.
 * 
 * Example:
 * 
 * ResourceFactory importer = ResourceFactoryRegister.getFactory("a1.h");
 * Resource importerResource = importer.load(resourceName);
 *  
 */
public class CImporter extends org.eclipse.emf.ecore.resource.impl.ResourceFactoryImpl {

    public static final String copyright = "Licensed Material - Property of IBM\n com.ibm.etools.c\n(C) Copyright IBM Corp. 2004 - All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp."; //$NON-NLS-1$

    private static CImporter _thisImporter = null;

    //	private String _uri;
    //	private ArrayList _includePaths;
    //	private CModelConverter _currentModelConverter;
    //	private DataStore _dataStore = null;
    //
    //	// Factories for the C TDLang model
    //	private CFactory _factory;
    //	private DatatypesFactory _datatypesFactory;

    /**
     * Constructor
     */
    public CImporter() {
        super();
        //	_uri = null;
        //
        //	// Initialize the C Package if running from the command line
        //	// If running within the Eclipse workbench, the packages are
        // initialized via plugin.xml.
        //	//
        //	if (!BootLoader.isRunning()) {
        //	 CPackageImpl.init();
        //	}
        //
        //	//CPackage cPkg =
        // (CPackage)RefRegister.getPackage(CPackage.packageURI);
        //	//DatatypesPackage dtPkg =
        // (DatatypesPackage)RefRegister.getPackage(DatatypesPackage.packageURI);
        //	org.eclipse.emf.ecore.EPackage.Registry registry =
        // org.eclipse.emf.ecore.EPackage.Registry.INSTANCE;
        //	CPackage cPkg = (CPackage)registry.getEPackage(CPackage.eNS_URI);
        //	DatatypesPackage dtPkg =
        // (DatatypesPackage)registry.getEPackage(DatatypesPackage.eNS_URI);
        //	
        //	_factory = cPkg.getCFactory();
        //	_datatypesFactory = dtPkg.getDatatypesFactory();
        //	_currentModelConverter = new CModelConverter(this);
        //	_includePaths = new ArrayList();
    }

    /**
     * Return the single instance of the CImporter
     */
    public static CImporter instance() {

        if (_thisImporter == null) {
            _thisImporter = new CImporter();
        }

        return _thisImporter;
    }

    // /**
    //  * Return the C Factory used to create C model elements
    //  */
    // public CFactory getFactory() {
    //   return _factory;
    // }
    //
    // /**
    //  * Return the C DataTypes Factory used to create C datatype elements
    //  */
    // public DatatypesFactory getDatatypesFactory() {
    //   return _datatypesFactory;
    // }
    //
    // /**
    //  * Return the current CModelConverter object
    //  */
    // public CModelConverter getCurrentModelConverter() {
    //  return _currentModelConverter;
    // }
    //
    // /**
    //  * Parse the include paths from a string separated by the pathSeparator
    //  * and store each path element into an array list.
    //  * @param pathString The string that represents the include paths
    //  * @param includePaths The array which stores the parsed result
    //  */
    // private void parseIncludePath(String pathString, ArrayList includePaths)
    // {
    //
    //   if (pathString == null || pathString.length() == 0)
    //    return;
    //
    //   if (pathString.indexOf(File.pathSeparatorChar) == -1)
    //    includePaths.add(pathString);
    //   else {
    //    StringTokenizer tokenizer = new StringTokenizer(pathString,
    // File.pathSeparator);
    //    while (tokenizer.hasMoreTokens()) {
    //     includePaths.add(tokenizer.nextToken());
    //    }
    //   }
    //
    // }
    //
    // /**
    //  * Return an array list that contains all the include files of the given
    // source file
    //  */
    // public ArrayList getIncludeFiles(String fileName) throws Exception {
    //
    //   String fullName = new String(fileName);
    //   File file = new File(fileName);
    //   if (file.exists()) {
    //     fullName = file.getAbsolutePath();
    //   }
    //   else {
    //     throw new CException(CResource.getString("_ERROR_File_Not_Exist",
    // fileName));
    //   }
    //
    //   CSourceUnit sourceUnit =
    // _currentModelConverter.getSourceUnitFromName(fullName);
    //   if (sourceUnit != null) {
    //     return sourceUnit.getIncludeFiles();
    //   }
    //   else {
    //     return null;
    //   }
    // }
    //
    // /**
    //  * Translate the parsed output saved in the DataElement tree
    //  * into MOF resource format.
    //  * The return result is a MOF resource object whose Extent is populated
    //  * with the parsed object list.
    //  * @param parsed_source com.ibm.cpp.core.model.DataElement
    //  * a data element representing the root node of the parsed tree
    //  * @return a resource object with an extent which is a MOF object list
    //  */
    // public Resource load(DataElement parsed_source) throws Exception {
    //
    //  ArrayList objectList =
    // _currentModelConverter.importSource(parsed_source);
    //
    //  //System.out.println("Size before expand: " + objectList.size());
    //  // Call the utility class to generate the full object list
    //  // The default implementaion is to generate a full object list. The full
    //  // list can be saved persistently to a disk file. The partial list will
    //  // suffice if you only use it in memory.
    //  //
    //  ArrayList finalObjectList = objectList;
    //  if (CPreferenceStore.isGenerateFullObjectList()) {
    //    finalObjectList = CUtility.generateFullObjectList(objectList);
    //  }
    //
    //  //System.out.println("Size after expand: " + finalObjectList.size());
    //  //Extent extent = new com.ibm.etools.emf.ref.impl.ExtentImpl();
    //  //extent.addAll(finalObjectList);
    //  //Resource outputResource = new
    // com.ibm.xmi.base.impl.XMIResourceImpl(_uri, extent);
    //  URI uri = URI.createFileURI(_uri);
    //  Resource outputResource = new
    // org.eclipse.emf.ecore.xmi.impl.XMIResourceImpl(uri);
    //  outputResource.getContents().addAll(finalObjectList);
    //  
    //  return outputResource;
    // }

    // /**
    //  * Retrieve the named type from the parsed file
    //  * Only the objects related to that type are added to the returned list
    //  */
    // public Resource load(DataElement parsed_source, String typeName) throws
    // Exception {
    //
    //  if (_currentModelConverter.getSourceUnitFromName(parsed_source.getName())
    // == null) {
    //    _currentModelConverter.importSource(parsed_source);
    //  }
    //
    //  ArrayList objectList = new ArrayList();
    //  CTypeTable typeTable = _currentModelConverter.getTypeTable();
    //  Object modelType = typeTable.getType(typeName);
    //
    //  if (modelType != null) {
    //   objectList.add(modelType);
    //  }
    //  else {
    //   // If we cannot find a type for the given name, try to find it as a
    // function
    //   Object modelFunction = typeTable.getFunction(typeName);
    //   if (modelFunction != null) {
    //    objectList.add(modelFunction);
    //   }
    //   else
    //    throw new CException(CResource.getString("_ERROR_No_Type_Or_Function",
    // typeName));
    //  }
    //
    //  ArrayList finalObjectList = objectList;
    //  if (CPreferenceStore.isGenerateFullObjectList()) {
    //    finalObjectList = CUtility.generateFullObjectList(objectList);
    //  }
    //
    //  //Extent extent = new com.ibm.etools.emf.ref.impl.ExtentImpl();
    //  //extent.addAll(finalObjectList);
    //  //Resource outputResource = new
    // com.ibm.xmi.base.impl.XMIResourceImpl(_uri, extent);
    //
    //  URI uri = URI.createFileURI(_uri);
    //  Resource outputResource = new
    // org.eclipse.emf.ecore.xmi.impl.XMIResourceImpl(uri);
    //  outputResource.getContents().addAll(finalObjectList);
    //
    //  return outputResource;
    // }

    ///**
    // * Loads the Resource with the given URI using the given load options.
    // * <p>
    // * The default behavior is to create an InputStream from the URI and
    // * invoke load(uri, inputstream, options). This method overloads the
    // * corresponding load method in ResourceFactoryImpl to correctly close
    // * the stream if an exception occurs.
    // */
    // public Resource load(String uri, Object options) throws Exception {
    //	URIConverter c = URIConverterRegister.getURIConverter();
    //	InputStream is = c.makeInputStream(uri);
    //	try {
    //		return load(uri, is, options);
    //	} finally {
    //		try {
    //			is.close();
    //		} catch (Exception e) {
    //		}
    //	}
    // }

    // /**
    //  * This is the only interface in ResourceFactory that we need to implement
    // in order to be
    //  * conformance with the importer framework. It is the entry pointer of the
    // importer.
    //  * This method is indirectly called via a call to
    // ResourceFactory.load(String uri).
    //  *
    //  * Here we initialize the DataStore, invoke the parser on the source file,
    //  * then call the other load(DataElement) method to translate the parsed
    // information into
    //  * MOF format.
    //  *
    //  * @return the Resource object
    //  * @param resources The ResourceSet
    //  * @param uri The URI representing the resource to parse
    //  * @param is The input stream
    //  * @param options The options
    //  */
    // public Resource load(ResourceSet resources, String uri, InputStream is,
    // Object options) throws Exception
    // {
    //
    //   initialize(options);
    //   
    //   _uri = uri;
    //
    //   // Throws an exception if the uri is null.
    //   if (uri == null || uri.length() == 0) {
    //    throw new CException(CResource.getString("_ERROR_URI_Is_Null"));
    //   }
    //
    //   // Get the URIFactory
    //   URIFactory uriFactory = URIFactoryRegister.getFactory();
    //   if (uriFactory == null) {
    //    throw new CException(CResource.getString("_ERROR_URIFactory_Is_Null"));
    //   }
    //
    //   String fileName = null;
    //   String typeName = null;
    //   try {
    //    URI myURI = uriFactory.makeURI(uri);
    //    URL myURL = myURI.getURL();
    //    // If the URL protocol is "platform", we need to resolve it to
    //    // the physical path.
    //    if (myURL.getProtocol().equals("platform")) {
    //     myURL = Platform.resolve(myURL);
    //    }
    //    fileName = myURL.getFile();
    //    typeName = myURL.getRef();
    //   }
    //   catch (Exception e) {
    //    // If uri is not in a valid format of URI, treat it as a
    //    // regular file name.
    //    fileName = new String(_uri);
    //    int index = _uri.indexOf('#');
    //    if (index > 0) {
    //     fileName = _uri.substring(0, index);
    //	 typeName = _uri.substring(index+1);
    //    }
    //   }
    //
    //   // Get the absolute path name
    //   File file = new File(fileName);
    //   if (file.exists()) {
    //     fileName = file.getAbsolutePath();
    //   }
    //   else {
    //     throw new CException(CResource.getString("_ERROR_File_Not_Exist",
    // fileName));
    //   }
    //
    //   // Create a client connection which will create a DataStore implicitly.
    //   ClientConnection client = new ClientConnection("Import");
    //   _dataStore = client.getDataStore();
    //
    //   // Set the plugin path which is used to locate the data files (e.g.
    // minerFile.dat)
    //   // needed by the cpp model.
    //   // This file is located at %eclipsedir%/plugins/com.ibm.etools.c/.
    //   //
    //   String pluginPath = CPlugin.getInstallationDirectory();
    // //org.eclipse.core.boot.BootLoader.getInstallURL().getFile();
    //// if (baseDir != null) {
    //// pluginPath = baseDir + "plugins/";
    //// }
    //// else {
    //// String eclipseDir = System.getProperty("eclipsedir");
    //// if (eclipseDir == null) {
    //// eclipseDir = "c:/eclipse";
    //// }
    //// pluginPath = eclipseDir + "/plugins/";
    //// }
    //
    //   _dataStore.setAttribute(DataStoreAttributes.A_PLUGIN_PATH, pluginPath);
    //   // the pluginpath now has everything it needs - set the location to blank
    //   _dataStore.setMinersLocation(""); //"com.ibm.etools.c");
    //   client.localConnect();
    //
    //   // Initialize the schema and miners
    //   _dataStore.showTicket(null);
    //   _dataStore.getSchema();
    //   _dataStore.initMiners();
    //
    //   // Create a DataElement to represent the object file we are going to
    // parse.
    //   DataElement parseFile = _dataStore.createObject(_dataStore.getHostRoot(),
    // "file", fileName, fileName);
    //
    //   // Find the miner root under which the parsed information is located.
    //   DataElement minerData =
    // _dataStore.findMinerInformation("com.ibm.cpp.miners.parser.ParseMiner");
    //
    //   // Create a fake project to hold the file
    //   DataElement theProject = _dataStore.createObject(minerData,
    // ParserSchema.Project, "fake project");
    //   DataElement parsedFiles =
    // _dataStore.createObject(theProject,ParserSchema.ParsedFiles,
    // ParserSchema.ParsedFiles);
    //   _dataStore.createObject(theProject,ParserSchema.SystemObjects,
    // ParserSchema.SystemObjects);
    //   _dataStore.createObject(theProject,ParserSchema.SystemObjects,
    // ParserSchema.ProjectObjects);
    //
    //   DataElement projectMinerProject = _dataStore.createObject(minerData,
    // ParserSchema.Project, "fake project2");
    //   _dataStore.createReference(projectMinerProject, theProject,
    // ParserSchema.ParseReference);
    //
    //
    //   // Set the include path
    //   DataElement preferences = _dataStore.createObject(theProject,
    // "preferences", "Preferences");
    //   DataElement includeElement = _dataStore.createObject(preferences,
    // "preferences", "Include Path");
    //   for (int i = 0; i < _includePaths.size(); i++)
    //    _dataStore.createObject(includeElement, "directory",
    // (String)_includePaths.get(i), (String)_includePaths.get(i));
    //
    //   // Invoke the C/C++ parser on the project
    //   ArrayList args = new ArrayList();
    //   args.add(projectMinerProject);
    //   DataElement status = _dataStore.command(ParserSchema.dC_PARSE, args,
    // parseFile);
    //
    //   // The parsing job is done on a service thread. We have to wait until
    //   // the parsing is finished.
    //   while (!status.getAttribute(DE.A_VALUE).equals("really done")) {
    //    Thread.currentThread().sleep(100);
    //   }
    //   
    //   // Retrieve the parsed source object under which the parsed information
    // is contained.
    //   DataElement files = _dataStore.find(theProject, DE.A_NAME,
    // ParserSchema.ParsedFiles, 2);
    //   String realFileName = null;
    //   try {
    //    realFileName = (new File(fileName)).getCanonicalPath();
    //   }
    //   catch (IOException e) {
    //    CResource.writeLog(CImporter.instance(), "load", e);
    //   }
    //
    //   DataElement parsedSource = _dataStore.find(files, DE.A_VALUE,
    // realFileName, 1);
    //
    //   // Terminate the parser background threads
    //   DataElement terminateParseThread = _dataStore.createObject(preferences,
    // "preferences", "Terminate Thread");
    //   DataElement cancelCommand =
    // _dataStore.find(_dataStore.getDescriptorRoot(), DE.A_VALUE, "C_CANCEL",
    // 2);
    //   _dataStore.command(cancelCommand, terminateParseThread, parseFile, true);
    //   
    //   // Throw an exception if the parsed source is null.
    //   if (parsedSource == null) {
    //     throw new CException(CResource.getString("_ERROR_Parsed_Source_Is_Null",
    // realFileName));
    //   }
    //   
    //   // Depending on whether a type name is specified, call the corresponding
    // load() method
    //   // to translate the parsed information to MOF format.
    //   Resource resource = null;
    //   if (typeName == null)
    //     resource = load(parsedSource);
    //   else
    //     resource = load(parsedSource, typeName);
    //   
    //   // Disconnect from the datastore
    //   client.disconnect();
    //   
    //   return resource;
    // }
    // 
    // /**
    //  * Initialize the values that are unique for each importer session.
    //  * These include the preference settings.
    //  */
    // private void initialize(Object options) throws Exception {
    // 
    //   CPreferenceStore.getValuesFromPlugin();
    //  
    //   // Parse the options if it is not null.
    //   if (options != null) {
    //    // Parse the options as a String.
    //    if (options instanceof String)
    //     CPreferenceStore.parseOptionsAsString((String)options);
    //    // Parse the options as key-value pairs
    //    else if (options instanceof HashMap)
    //     CPreferenceStore.parseOptionsAsKey((HashMap)options);
    //   }
    //   
    //   // Create the platform compiler info object
    //   CTypeDescriptorBuilder.createPlatformCompilerInfo();
    //   // The Type Descriptor map is unique for each importer session.
    //   CPrimitiveTypesMapper.createTypeDescriptorMap();
    //   // Set the bitfield layout rule
    //   CBitFieldHelper.setRule();
    //   
    //   // Parse the include paths
    //   _includePaths.clear();
    //   parseIncludePath(CPreferenceStore.getIncludePaths(), _includePaths);
    //
    //   // By default, we use a new CModelConverter
    //   // object for every importer session. If the "-sameproject" option is
    // specified,
    //   // we will reuse the existing CModelConverter object and the existing
    // namespace.
    //   //
    //   if (!CPreferenceStore.isReuseParseInformation())
    //    _currentModelConverter = new CModelConverter(this);
    // 
    // }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.emf.ecore.resource.impl.ResourceFactoryImpl#createResource(org.eclipse.emf.common.util.URI)
     */
    public Resource createResource(URI arg0) {
        return new CImporterResource(arg0);
    }

}